#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

unsigned int BASE = 0x300;

#define uint unsigned int
#define PI 3.1415926535897932384626433832795


void CtrMode(uint addr, char cntr, char mode)
{
  int ctrl;
  ctrl = (cntr << 6) | 0x30 | (mode << 1);
  outportb(addr+3,ctrl);
}

void CtrLoad(uint addr ,int c,int val)
{
  outportb(addr+c,val & 0x00FF);
  outportb(addr+c,(val>>8) & 0x00FF);
}

unsigned AskForBaseAddress(unsigned int OldOne)
{
	char msg[8];
	int NewOne = 0, Success = 0, Dummy;
	int AddrInputPosX, AddrInputPosY;

	puts("Please enter the Base Address (100-FFFF) for your card (in hex)");
	printf("or press ENTER for %X.\n>", OldOne);
	AddrInputPosX = wherex(); AddrInputPosY = wherey();
	do {
		gotoxy(AddrInputPosX, AddrInputPosY);
		clreol();
		msg[0] = 5; msg[1] = 0;
		cgets(msg);
		sscanf(msg + 2, "%x", &NewOne);
		if (NewOne >= 0x100)
		{
			Success = 1;
			Dummy = NewOne;
		} else if (msg[1] == 0) {
			gotoxy(AddrInputPosX, AddrInputPosY);
			printf("%X", OldOne);
			Success = 1;
			Dummy = OldOne;
		}
	} while(!Success);
	return (Dummy);
} /* end of AskForBaseAddress */


void intro ()
{
    clrscr();
    puts("This program will output six wave forms from the 104-DA12-8.");
    puts("The wave forms will be output on DACS 0 - 5.");
    puts("The wave forms will stop when the program exits.");
}


void main (void)
{
    long int count;
    long int NumSamples = 6 * 10000;

    intro();

    BASE = AskForBaseAddress(BASE);
    printf("Address Selected: %04X",BASE);
    /*****************************************************
    ** This for loop will load the board with sine waves,
    ** with an amplitude of half scale, rising from the
    ** bottom half to the top half of the range. Each
    ** channel has a different frequency.
    *****************************************************/



    //now fill the SRAM on the card
    for (count = 0; count < NumSamples; count++)
    {
        unsigned short ThisSample;
        switch ( count % 6 )
        {
            case 0:
            {
                ThisSample = (sin(count * (2.0 * PI * 1.0 / 1000.0)) + 1.0 + count * 2.0 / NumSamples) * 4095.0/4.0;
                ThisSample &= 0x0FFF;
                break;
            }
            case 1:
            {
                ThisSample = (sin(count * (2.0 * PI * 2.0 / 1000.0)) + 1.0 + count * 2.0 / NumSamples) * 4095.0/4.0;
                ThisSample &= 0x0FFF;
                break;
            }
            case 2:
            {
                ThisSample = (sin(count * (2.0 * PI * 3.0 / 1000.0)) + 1.0 + count * 2.0 / NumSamples) * 4095.0/4.0;
                ThisSample &= 0x0FFF;
                break;
            }
            case 3:
            {
                ThisSample = (sin(count * (2.0 * PI * 4.0 / 1000.0)) + 1.0 + count * 2.0 / NumSamples) * 4095.0/4.0;
                ThisSample &= 0x0FFF;
                break;
            }
            case 4:
            {
                ThisSample = (sin(count * (2.0 * PI * 5.0 / 1000.0)) + 1.0 + count * 2.0 / NumSamples) * 4095.0/4.0;
                ThisSample &= 0x0FFF;
                break;
            }
            case 5:
            {
                ThisSample = (sin(count * (2.0 * PI * 6.0 / 1000.0)) + 1.0 + count * 2.0 / NumSamples) * 4095.0/4.0;
                ThisSample &= 0x0FFF;
                //This is the last channel, so set end-of-DACs
                ThisSample |= 0x2000;
                break;
            }
        }
        //this is the end of the stream, so set loop
        if ( count == NumSamples - 1 ) ThisSample |= 0x2000;

        //set the address we are going to write to
        outport(BASE + 0x18, (count * 2));
        outportb(BASE + 0x1A, (count * 2) >> 16);
        //write the value for that address
        outport(BASE + 0x1c, ThisSample);
    }


    CtrMode(BASE + 0x14, 1, 2); //set counter 1 to mode 2
    CtrMode(BASE + 0x14, 2, 2); //set counter 2 to mode 2

    CtrLoad(BASE + 0x14, 1, 5); //load counter 1 to 5 ticks
    CtrLoad(BASE + 0x14, 2, 10); //load counter 2 to 10 ticks



    outportb(BASE + 0x10, 0x41); //tell the card to start and enable the ARB


    puts("\nPress any key to stop");

    while ( ! kbhit() ) ;

    outportb(BASE + 0x10, 0); //tell the card to stop

    clrscr();

}